﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;

namespace TestProject
{
    /// <summary>
    /// 물고기 눈 컨트롤
    /// </summary>
    public class FishEyeControl : StackPanel
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Dependency Property
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 애니메이션 밀리초 속성 - AnimationMillisecondsProperty

        /// <summary>
        /// 애니메이션 밀리초 속성
        /// </summary>
        public static readonly DependencyProperty AnimationMillisecondsProperty = DependencyProperty.Register
        (
            "AnimationMilliseconds",
            typeof(int),
            typeof(FishEyeControl),
            new UIPropertyMetadata(65)
        );

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 엘리먼트 너비
        /// </summary>
        private double elementWidth;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 애니메이션 밀리초 - AnimationMilliseconds

        /// <summary>
        /// 애니메이션 밀리초
        /// </summary>
        public int AnimationMilliseconds
        {
            get
            {
                return (int)GetValue(AnimationMillisecondsProperty);
            }
            set
            {
                SetValue(AnimationMillisecondsProperty, value);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - FishEyeControl()

        /// <summary>
        /// 생성자
        /// </summary>
        public FishEyeControl()
        {
            HorizontalAlignment = HorizontalAlignment.Center;
            Background          = Brushes.Transparent;
            Orientation         = Orientation.Horizontal;

            Loaded     += StackPanel_Loaded;
            MouseEnter += StackPanel_MouseEnter;
            MouseMove  += StackPanel_MouseMove;
            MouseLeave += StackPanel_MouseLeave;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 측정하기 (오버라이드) - MeasureOverride(availableSize)

        /// <summary>
        /// 측정하기 (오버라이드)
        /// </summary>
        /// <param name="availableSize">이용 가능한 크기</param>
        /// <returns>측정 크기</returns>
        protected override Size MeasureOverride(Size availableSize)
        {
            UIElementCollection childCollection = InternalChildren;

            Size size  = new Size();
            int  i     = 0;
            int  count = childCollection.Count;

            availableSize.Width = double.PositiveInfinity;

            while(i < count)
            {
                UIElement element = childCollection[i];

                if(element != null)
                {
                    element.Measure((Size)availableSize);

                    Size desiredSize = element.DesiredSize;

                    size.Width += Math.Round(desiredSize.Width, 2);

                    size.Height = Math.Max(size.Height, desiredSize.Height);
                }

                i++;
            }

            return size;
        }

        #endregion
        #region 배열하기 (오버라이드) - ArrangeOverride(arrangeSize)

        /// <summary>
        /// 배열하기 (오버라이드)
        /// </summary>
        /// <param name="arrangeSize">배열 크기</param>
        /// <returns>배열 크기</returns>
        protected override Size ArrangeOverride(Size arrangeSize)
        {
            if(Children == null || Children.Count == 0)
            {
                return arrangeSize;
            }

            UIElementCollection childCollection = Children;

            Rect finalRectangle = new Rect(arrangeSize);

            double width = 0d;
            int    i     = 0;
            int    count = childCollection.Count;

            while(i < count)
            {
                UIElement element = childCollection[i];

                if(element != null)
                {
                    finalRectangle.X += width;

                    width = element.DesiredSize.Width;

                    finalRectangle.Width  = width;
                    finalRectangle.Height = Math.Max(arrangeSize.Height, element.DesiredSize.Height);

                    element.SetValue(WidthProperty, element.DesiredSize.Width);

                    if(element.RenderTransform as TransformGroup == null)
                    {
                        TransformGroup transformGroup = new TransformGroup();

                        element.RenderTransform = transformGroup;

                        transformGroup.Children.Add(new ScaleTransform()    );
                        transformGroup.Children.Add(new TranslateTransform());

                        element.RenderTransformOrigin = new Point(0.5, 1);
                    }

                    element.Arrange(finalRectangle);
                }

                i++;
            }

            AnimateAll();

            return arrangeSize;
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 스택 패널 로드시 처리하기 - StackPanel_Loaded(sender, e)

        /// <summary>
        /// 스택 패널 로드시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void StackPanel_Loaded(object sender, RoutedEventArgs e)
        {
            if(Children == null || Children.Count == 0)
            {
                return;
            }

            foreach(UIElement child in Children)
            {
                this.elementWidth += child.DesiredSize.Width;
            }

            this.elementWidth /= Children.Count;

            InvalidateArrange();
        }

        #endregion
        #region 스택 패널 마우스 진입시 처리하기 - StackPanel_MouseEnter(sender, e)

        /// <summary>
        /// 스택 패널 마우스 진입시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void StackPanel_MouseEnter(object sender, MouseEventArgs e)
        {
            InvalidateArrange();
        }

        #endregion
        #region 스택 패널 마우스 이동시 처리하기 - StackPanel_MouseMove(sender, e)

        /// <summary>
        /// 스택 패널 마우스 이동시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void StackPanel_MouseMove(object sender, MouseEventArgs e)
        {
            InvalidateArrange();
        }

        #endregion
        #region 스택 패널 마우스 이탈시 처리하기 - StackPanel_MouseLeave(sender, e)

        /// <summary>
        /// 스택 패널 마우스 이탈시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void StackPanel_MouseLeave(object sender, MouseEventArgs e)
        {
            InvalidateArrange();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 애니메이션 구하기 - GetAnimation(to, duration)

        /// <summary>
        /// 애니메이션 구하기
        /// </summary>
        /// <param name="to">목표</param>
        /// <param name="duration">지속 시간</param>
        /// <returns>실수 애니메이션</returns>
        private DoubleAnimation GetAnimation(double to, double duration)
        {
            DoubleAnimation animation = new DoubleAnimation(to, TimeSpan.FromMilliseconds(duration))
            {
                AccelerationRatio = 0.2,
                DecelerationRatio = 0.1
            };

            return animation;
        }

        #endregion
        #region 애니메이션 처리하기 - AnimateTo(child, scale, width, duration)

        /// <summary>
        /// 애니메이션 처리하기
        /// </summary>
        /// <param name="child">자식</param>
        /// <param name="scale">스케일</param>
        /// <param name="width">너비</param>
        /// <param name="duration">지속 시간</param>
        private void AnimateTo(UIElement child, double scale, double width, double duration)
        {
            TransformGroup transformGroup = child.RenderTransform as TransformGroup;

            ScaleTransform scaleTransform = transformGroup.Children[0] as ScaleTransform;

            scaleTransform.BeginAnimation(ScaleTransform.ScaleXProperty, GetAnimation(scale, duration));
            scaleTransform.BeginAnimation(ScaleTransform.ScaleYProperty, GetAnimation(scale, duration));

            child.BeginAnimation(WidthProperty, GetAnimation(width, duration));
        }

        #endregion
        #region 모두 애니메이션 처리하기 - AnimateAll()

        /// <summary>
        /// 모두 애니메이션 처리하기
        /// </summary>
        private void AnimateAll()
        {
            if(Children == null || Children.Count == 0)
            {
                return;
            }

            int? selectedElement = null;

            if(IsMouseOver)
            {
                double width = 0d;
                double x     = Mouse.GetPosition(this).X;
                double y     = Mouse.GetPosition(this).Y;

                for(int i = 0; i < Children.Count; i++)
                {
                    if(Children[i].IsMouseOver && y < Children[i].DesiredSize.Height)
                    {
                        selectedElement = i;

                        break;
                    }

                    width += Children[i].DesiredSize.Width;

                    if(x <= width && y < Children[i].DesiredSize.Height)
                    {
                        selectedElement = i;
                        break;
                    }
                }
            }

            for(int i = 0; i < Children.Count; i++)
            {
                if(i == selectedElement - 2)
                {
                    AnimateTo(Children[i], 1.25, (this.elementWidth * 1.5), AnimationMilliseconds);
                }
                else if(i == selectedElement - 1)
                {
                    AnimateTo(Children[i], 1.5, (this.elementWidth * 2.0), AnimationMilliseconds);
                }
                else if(i == selectedElement)
                {
                    AnimateTo(Children[i], 2, (this.elementWidth * 2.5), AnimationMilliseconds);
                }
                else if(i == selectedElement + 1)
                {
                    AnimateTo(Children[i], 1.5, (this.elementWidth * 2.0), AnimationMilliseconds);
                }
                else if(i == selectedElement + 2)
                {
                    AnimateTo(Children[i], 1.25, (this.elementWidth * 1.5), AnimationMilliseconds);
                }
                else
                {
                    AnimateTo(Children[i], 1, this.elementWidth, AnimationMilliseconds);
                }
            }
        }

        #endregion
    }
}