﻿using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;

namespace TestProject
{
    /// <summary>
    /// 반사 컨트롤
    /// </summary>
    public class ReflectionControl : Decorator
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 반사 비주얼 브러시
        /// </summary>
        private readonly VisualBrush reflectionVisualBrush;

        /// <summary>
        /// 불투명도 마스크 선형 그라디언트 브러시
        /// </summary>
        private readonly LinearGradientBrush opacityMaskLinearGradientBrush;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - ReflectionControl()

        /// <summary>
        /// 생성자
        /// </summary>
        public ReflectionControl()
        {
            HorizontalAlignment = HorizontalAlignment.Center;
            VerticalAlignment   = VerticalAlignment.Bottom;

            this.opacityMaskLinearGradientBrush = new LinearGradientBrush
            {
                StartPoint = new Point(0, 0  ),
                EndPoint   = new Point(0, 0.8)
            };

            this.opacityMaskLinearGradientBrush.GradientStops.Add(new GradientStop(Colors.Black      , 0  ));
            this.opacityMaskLinearGradientBrush.GradientStops.Add(new GradientStop(Colors.Black      , 0.5));
            this.opacityMaskLinearGradientBrush.GradientStops.Add(new GradientStop(Colors.Transparent, 0.8));
            this.opacityMaskLinearGradientBrush.GradientStops.Add(new GradientStop(Colors.Transparent, 1  ));

            this.reflectionVisualBrush = new VisualBrush
            {
                Transform = new ScaleTransform(1, -1)
            };
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 측정하기 (오버라이드) - MeasureOverride(availableSize)

        /// <summary>
        /// 측정하기 (오버라이드)
        /// </summary>
        /// <param name="availableSize">이용 가능한 크기</param>
        /// <returns>측정 크기</returns>
        protected override Size MeasureOverride(Size availableSize)
        {
            if(Child == null)
            {
                return new Size(0, 0);
            }

            Child.Measure(availableSize);

            return new Size(Child.DesiredSize.Width, Child.DesiredSize.Height * 2.6);
        }

        #endregion
        #region 배열하기 (오버라이드) - ArrangeOverride(finalSize)

        /// <summary>
        /// 배열하기 (오버라이드)
        /// </summary>
        /// <param name="finalSize">최종 크기</param>
        /// <returns>배열 크기</returns>
        protected override Size ArrangeOverride(Size finalSize)
        {
            if(Child == null)
            {
                return new Size(0, 0);
            }

            Child.Arrange(new Rect(0, -(finalSize.Height * 2), finalSize.Width, finalSize.Height * 2.6));

            return finalSize;
        }

        #endregion
        #region 렌더링시 처리하기 - OnRender(drawingContext)

        /// <summary>
        /// 렌더링시 처리하기
        /// </summary>
        /// <param name="drawingContext">그리기 컨텍스트</param>
        protected override void OnRender(DrawingContext drawingContext)
        {
            base.OnRender(drawingContext);

            drawingContext.PushOpacityMask(this.opacityMaskLinearGradientBrush);

            drawingContext.PushOpacity(0.3);

            this.reflectionVisualBrush.Visual = Child;

            ((ScaleTransform)this.reflectionVisualBrush.Transform).CenterY = 4 * ActualHeight / 5;

            drawingContext.DrawRectangle
            (
                this.reflectionVisualBrush,
                null,
                new Rect(0, ActualHeight / 2, ActualWidth, ActualHeight / 2)
            );

            drawingContext.Pop();
            drawingContext.Pop();
        }

        #endregion
    }
}